"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RoutingService = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@angular/core");
const router_1 = require("@angular/router");
const common_1 = require("@angular/common");
let skipNavigation = false;
const noop = function () { };
const navigateByUrl = router_1.Router.prototype.navigateByUrl;
// deactivates the usual platform behavior; all these operations are performed via the RoutingService
// to avoid any conflict, e.g., double-booking URL changes in React and Angular
common_1.BrowserPlatformLocation.prototype.pushState = noop;
common_1.BrowserPlatformLocation.prototype.replaceState = noop;
common_1.BrowserPlatformLocation.prototype.forward = noop;
common_1.BrowserPlatformLocation.prototype.back = noop;
common_1.BrowserPlatformLocation.prototype.historyGo = noop;
// required to detect / react to hidden URL change (see #554 for details)
router_1.Router.prototype.navigateByUrl = function (url, extras) {
    skipNavigation = extras?.skipLocationChange ?? false;
    const result = navigateByUrl.call(this, url, extras);
    skipNavigation = false;
    return result;
};
function normalize(url) {
    const search = url.indexOf('?');
    const hash = url.indexOf('#');
    if (search !== -1 || hash !== -1) {
        if (search === -1) {
            return url.substring(0, hash);
        }
        else if (hash === -1) {
            return url.substring(0, search);
        }
        else {
            return url.substring(0, Math.min(search, hash));
        }
    }
    return url;
}
let RoutingService = class RoutingService {
    constructor(context, router, zone) {
        this.context = context;
        this.router = router;
        this.zone = zone;
        this.invalidRoutes = [];
        if (this.router) {
            this.router.errorHandler = (error) => {
                // Match in development and production
                if (error.message.match('Cannot match any routes') || error.message.match('NG04002')) {
                    // ignore this special error
                    return undefined;
                }
                throw error;
            };
            const skipIds = [];
            const nav = this.context.navigation;
            const queueNavigation = (url) => {
                window.requestAnimationFrame(() => nav.push(url));
            };
            this.dispose = nav.listen(({ location }) => {
                const path = location.pathname;
                if (!this.invalidRoutes.includes(path)) {
                    const url = `${path}${location.search}${location.hash}`;
                    this.zone.run(() => navigateByUrl.call(this.router, url));
                }
            });
            this.subscription = this.router.events.subscribe((e) => {
                if (e instanceof router_1.NavigationError) {
                    const routerUrl = e.url;
                    const path = normalize(routerUrl);
                    const locationUrl = nav.url;
                    if (!this.invalidRoutes.includes(path)) {
                        this.invalidRoutes.push(path);
                    }
                    if (routerUrl !== locationUrl) {
                        queueNavigation(routerUrl);
                    }
                }
                else if (e.type === 0 && skipNavigation) {
                    skipIds.push(e.id);
                }
                else if (e.type === 15) {
                    const index = skipIds.indexOf(e.routerEvent.id);
                    if (index === -1) {
                        // consistency check to avoid #535 and other Angular-specific issues
                        const locationUrl = nav.url;
                        const routerUrl = e.routerEvent.url;
                        if (routerUrl !== locationUrl) {
                            queueNavigation(routerUrl);
                        }
                    }
                    else {
                        skipIds.splice(index, 1);
                    }
                }
            });
        }
    }
    ngOnDestroy() {
        this.dispose?.();
        this.subscription?.unsubscribe();
    }
};
exports.RoutingService = RoutingService;
exports.RoutingService = RoutingService = tslib_1.__decorate([
    (0, core_1.Injectable)(),
    tslib_1.__param(0, (0, core_1.Inject)('Context')),
    tslib_1.__param(1, (0, core_1.Optional)()),
    tslib_1.__param(2, (0, core_1.Optional)()),
    tslib_1.__metadata("design:paramtypes", [Object, router_1.Router,
        core_1.NgZone])
], RoutingService);
//# sourceMappingURL=RoutingService.js.map