"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bootstrap = exports.prepareBootstrap = void 0;
const module_1 = require("./module");
const utils_1 = require("./utils");
const startup_1 = require("./startup");
async function prepareBootstrap(moduleOrComponent, piral) {
    if ('module' in moduleOrComponent && typeof moduleOrComponent.module === 'function') {
        if (!(moduleOrComponent.state.current instanceof Promise)) {
            moduleOrComponent.state.current = moduleOrComponent.module().then((result) => {
                if (typeof result !== 'object' || !('default' in result)) {
                    throw new Error('The lazy loaded module does not `default` export a NgModule class.');
                }
                (0, module_1.defineModule)(result.default, moduleOrComponent.opts, moduleOrComponent.flags);
                return (0, module_1.findModule)(result.default);
            });
        }
        const moduleDef = await moduleOrComponent.state.current;
        const { components } = moduleDef;
        const component = components.find((m) => (0, utils_1.hasSelector)(m, moduleOrComponent.selector));
        if (!component) {
            throw new Error(`No component matching the selector "${moduleOrComponent.selector}" has been found.`);
        }
        return [...(0, module_1.activateModuleInstance)(moduleDef, piral), component];
    }
    else {
        const [annotation] = (0, utils_1.getAnnotations)(moduleOrComponent);
        const standalone = annotation?.standalone;
        // first way is to directly use a module, which is the legacy way
        // second way is to find a previously defined Angular module
        if (annotation && annotation.bootstrap) {
            // usually contains things like imports, exports, declarations, ...
            const [component] = annotation.bootstrap;
            annotation.exports = [component];
            (0, module_1.defineModule)(moduleOrComponent);
            return [...(0, module_1.getModuleInstance)(component, standalone, piral), component];
        }
        else {
            // usually contains things like selector, template or templateUrl, changeDetection, ...
            const result = (0, module_1.getModuleInstance)(moduleOrComponent, standalone, piral) ||
                (0, module_1.createModuleInstance)(moduleOrComponent, standalone, piral);
            return [...result, moduleOrComponent];
        }
    }
}
exports.prepareBootstrap = prepareBootstrap;
async function bootstrap(result, node, props, context) {
    const [selectedModule, ngOptions, ngFlags, component] = result;
    const ref = await (0, startup_1.startup)(selectedModule, context, ngOptions, ngFlags);
    if (ref) {
        ref.instance.attach(component, node, props);
        return () => ref.instance.detach(component, node);
    }
    return () => { };
}
exports.bootstrap = bootstrap;
//# sourceMappingURL=bootstrap.js.map