"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createConverter = void 0;
const rxjs_1 = require("rxjs");
const queue_1 = require("./queue");
const module_1 = require("./module");
const bootstrap_1 = require("./bootstrap");
const common_1 = require("../common");
function createConverter(_ = {}) {
    const registry = new Map();
    const convert = (component) => ({
        mount(el, props, ctx, locals) {
            locals.active = true;
            if (!locals.props) {
                locals.props = new rxjs_1.BehaviorSubject(props);
            }
            if (!locals.queued) {
                locals.queued = Promise.resolve();
            }
            locals.queued = locals.queued.then(() => (0, queue_1.enqueue)(async () => {
                if (!registry.has(component)) {
                    registry.set(component, await (0, bootstrap_1.prepareBootstrap)(component, props.piral));
                }
                if (locals.active) {
                    return await (0, bootstrap_1.bootstrap)(registry.get(component), el, locals.props, ctx);
                }
            }));
        },
        update(el, props, ctx, locals) {
            locals.props.next(props);
        },
        unmount(el, locals) {
            locals.active = false;
            locals.queued = locals.queued.then((dispose) => dispose && (0, queue_1.enqueue)(dispose));
        },
    });
    convert.defineModule = module_1.defineModule;
    convert.Extension = common_1.NgExtension;
    return convert;
}
exports.createConverter = createConverter;
//# sourceMappingURL=converter.js.map