"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defineModule = exports.findModule = exports.createModuleInstance = exports.getModuleInstance = exports.activateModuleInstance = void 0;
const tslib_1 = require("tslib");
const platform_browser_1 = require("@angular/platform-browser");
const common_1 = require("@angular/common");
const core_1 = require("@angular/core");
const startup_1 = require("./startup");
const RoutingService_1 = require("./RoutingService");
const utils_1 = require("./utils");
const common_2 = require("../common");
const availableModules = [];
function instantiateModule(moduleDef, piral) {
    var BootstrapModule_1;
    const { module, components } = moduleDef;
    const imports = [platform_browser_1.BrowserModule, common_2.SharedModule, module];
    const props = { current: undefined };
    const createProxy = () => new Proxy(props.current, {
        get(_, name) {
            return props.current[name];
        },
    });
    const providers = [
        RoutingService_1.RoutingService,
        { provide: 'Props', useFactory: createProxy, deps: [] },
        { provide: 'piral', useFactory: () => piral, deps: [] },
    ];
    let BootstrapModule = BootstrapModule_1 = class BootstrapModule {
        constructor(resolver, zone, routing, flags) {
            this.resolver = resolver;
            this.zone = zone;
            this.routing = routing;
            this.flags = flags;
            this.refs = [];
        }
        ngDoBootstrap(appRef) {
            this.appRef = appRef;
        }
        attach(component, node, $props) {
            const factory = this.resolver.resolveComponentFactory(component);
            props.current = $props.value;
            if (factory) {
                const ref = this.zone.run(() => this.appRef.bootstrap(factory, node));
                const name = ref.componentType?.ɵcmp?.inputs?.Props;
                if (typeof name === 'string') {
                    const sub = $props.subscribe((props) => {
                        ref.instance[name] = props;
                        ref.changeDetectorRef?.detectChanges();
                    });
                    ref.onDestroy(() => sub.unsubscribe());
                }
                this.refs.push([component, node, ref]);
            }
        }
        detach(component, node) {
            for (let i = this.refs.length; i--;) {
                const [sourceComponent, sourceNode, ref] = this.refs[i];
                if (sourceComponent === component && sourceNode === node) {
                    ref.destroy();
                    this.refs.splice(i, 1);
                }
            }
            if (!this.flags?.keepAlive && this.refs.length === 0) {
                (0, startup_1.teardown)(BootstrapModule_1);
            }
        }
    };
    BootstrapModule = BootstrapModule_1 = tslib_1.__decorate([
        (0, core_1.NgModule)({
            imports,
            // @ts-ignore
            entryComponents: components,
            providers,
        }),
        tslib_1.__param(3, (0, core_1.Inject)('NgFlags')),
        tslib_1.__metadata("design:paramtypes", [core_1.ComponentFactoryResolver,
            core_1.NgZone,
            RoutingService_1.RoutingService, Object])
    ], BootstrapModule);
    return BootstrapModule;
}
function activateModuleInstance(moduleDef, piral) {
    if (!moduleDef.active) {
        moduleDef.active = instantiateModule(moduleDef, piral);
    }
    return [moduleDef.active, moduleDef.opts, moduleDef.flags];
}
exports.activateModuleInstance = activateModuleInstance;
function getModuleInstance(component, standalone, piral) {
    const [moduleDef] = availableModules.filter((m) => m.components.includes(component));
    if (moduleDef) {
        return activateModuleInstance(moduleDef, piral);
    }
    if (process.env.NODE_ENV === 'development') {
        if (!standalone) {
            console.warn('Component not found in all defined Angular modules. Make sure to define (using `defineNgModule`) a module with your component(s) referenced in the exports section of the `@NgModule` decorator.', component, piral.meta);
        }
    }
    return undefined;
}
exports.getModuleInstance = getModuleInstance;
function createModuleInstance(component, standalone, piral) {
    const declarations = standalone ? [] : [component];
    const importsDef = standalone ? [common_1.CommonModule, component] : [common_1.CommonModule];
    const exportsDef = [component];
    const schemasDef = [core_1.CUSTOM_ELEMENTS_SCHEMA];
    let Module = class Module {
    };
    Module = tslib_1.__decorate([
        (0, core_1.NgModule)({
            declarations,
            imports: importsDef,
            exports: exportsDef,
            schemas: schemasDef,
        })
    ], Module);
    defineModule(Module);
    return getModuleInstance(component, standalone, piral);
}
exports.createModuleInstance = createModuleInstance;
function findModule(module) {
    return availableModules.find((m) => m.module === module);
}
exports.findModule = findModule;
function defineModule(module, opts = undefined, flags = undefined) {
    const [annotation] = (0, utils_1.getAnnotations)(module);
    if (annotation) {
        availableModules.push({
            active: undefined,
            components: (0, utils_1.findComponents)(annotation.exports),
            module,
            flags,
            opts,
        });
    }
    else if (typeof module === 'function') {
        const state = {
            current: undefined,
        };
        return (selector) => ({
            component: { selector, module, opts, flags, state },
            type: 'ng',
        });
    }
}
exports.defineModule = defineModule;
//# sourceMappingURL=module.js.map