"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startup = exports.teardown = void 0;
const core_1 = require("@angular/core");
const common_1 = require("@angular/common");
const platform_browser_dynamic_1 = require("@angular/platform-browser-dynamic");
const utils_1 = require("./utils");
function getVersionHandler(versions) {
    const major = (0, utils_1.getNgVersion)();
    const version = `v${major}`;
    return versions[version];
}
// Equivalent to platformBrowserDynamic, but with support for multiple platforms
const customPlatformDynamicFactory = (0, core_1.createPlatformFactory)(platform_browser_dynamic_1.ɵplatformCoreDynamic, 'piralDynamic', [
    ...platform_browser_dynamic_1.ɵINTERNAL_BROWSER_DYNAMIC_PLATFORM_PROVIDERS,
    {
        provide: core_1.ɵALLOW_MULTIPLE_PLATFORMS,
        useValue: true,
    },
]);
const runningModules = [];
function startNew(BootstrapModule, context, ngOptions, ngFlags) {
    const path = context.publicPath || '/';
    const platform = customPlatformDynamicFactory([
        { provide: 'Context', useValue: context },
        { provide: common_1.APP_BASE_HREF, useValue: path },
        { provide: 'NgFlags', useValue: ngFlags },
    ]);
    // We need to bind the version-specific NgZone to its ID
    // this will not be MF-dependent, but Angular dependent
    // i.e., to allow using Zone.js with multiple versions of Angular
    const zoneIdentifier = `piral-ng:${core_1.VERSION.full}`;
    // This is a hack, since NgZone doesn't allow you to configure the property that identifies your zone.
    // See:
    // - https://github.com/PlaceMe-SAS/single-spa-angular-cli/issues/33
    // - https://github.com/angular/angular/blob/a14dc2d7a4821a19f20a9547053a5734798f541e/packages/core/src/zone/ng_zone.ts#L144
    // - https://github.com/angular/angular/blob/a14dc2d7a4821a19f20a9547053a5734798f541e/packages/core/src/zone/ng_zone.ts#L257
    // @ts-ignore
    core_1.NgZone.isInAngularZone = () => window.Zone.current._properties[zoneIdentifier] === true;
    // We disable those checks as they are misleading and might cause trouble
    core_1.NgZone.assertInAngularZone = () => { };
    core_1.NgZone.assertNotInAngularZone = () => { };
    return platform
        .bootstrapModule(BootstrapModule, ngOptions)
        .catch((err) => console.error(err))
        .then((instance) => {
        if (instance) {
            const zone = instance.injector.get(core_1.NgZone);
            // @ts-ignore
            const z = zone?._inner ?? zone?.inner;
            if (z && '_properties' in z) {
                z._properties[zoneIdentifier] = true;
            }
            runningModules.push([BootstrapModule, instance, platform]);
        }
        return instance;
    });
}
function teardown(BootstrapModule) {
    const runningModuleIndex = runningModules.findIndex(([ref]) => ref === BootstrapModule);
    if (runningModuleIndex !== -1) {
        const [, , platform] = runningModules[runningModuleIndex];
        runningModules.splice(runningModuleIndex, 1);
        if (!platform.destroyed) {
            platform.destroy();
        }
    }
}
exports.teardown = teardown;
function startup(BootstrapModule, context, ngOptions, ngFlags) {
    const runningModule = runningModules.find(([ref]) => ref === BootstrapModule);
    if (runningModule) {
        const [, instance, platform] = runningModule;
        if (platform.destroyed) {
            teardown(BootstrapModule);
        }
        else {
            return Promise.resolve(instance);
        }
    }
    return startNew(BootstrapModule, context, ngOptions, ngFlags);
}
exports.startup = startup;
if (process.env.NODE_ENV === 'development') {
    // May be used later for something useful. Right now only debugging output.
    const versionHandlers = {
        legacy() {
            console.log('Running in legacy mode (Angular 2-8)');
        },
        outdated() {
            console.log('Running in outdated mode (Angular 9-13)');
        },
        current() {
            console.log('Running in current mode (Angular 14-17)');
        },
        next() {
            console.log('Running in next mode (Angular 18)');
        },
        unknown() {
            console.log('Running with an unknown version of Angular');
        },
    };
    const versions = {
        v2: versionHandlers.legacy,
        v4: versionHandlers.legacy,
        v5: versionHandlers.legacy,
        v6: versionHandlers.legacy,
        v7: versionHandlers.legacy,
        v8: versionHandlers.legacy,
        v9: versionHandlers.outdated,
        v10: versionHandlers.outdated,
        v11: versionHandlers.outdated,
        v12: versionHandlers.outdated,
        v13: versionHandlers.outdated,
        v14: versionHandlers.current,
        v15: versionHandlers.current,
        v16: versionHandlers.current,
        v17: versionHandlers.current,
        v18: versionHandlers.next,
    };
    const handler = getVersionHandler(versions) || versionHandlers.unknown;
    handler();
}
if (process.env.NODE_ENV === 'production') {
    (0, core_1.enableProdMode)();
}
//# sourceMappingURL=startup.js.map